---- Asset manager
	require "entity"
	require "component"

	assetManager = {}
	
	assetManager.listGui = {}
	assetManager.listProps = {}
	assetManager.listAnimations = {}
	assetManager.listEntities = {}
	assetManager.listComponents = {}

-- Load gui graphics
	function assetManager.loadGui()
		print("Loading gui graphics")
		local listFileNames = love.filesystem.getDirectoryItems("art/gui/")
		
		for _ , fileName in pairs(listFileNames) do
			local nameKey = string.sub(fileName , 1 , #fileName - 4)
			local image = love.graphics.newImage("art/gui/" .. fileName)
				
			local sizeX = image:getWidth() / 1024
			local sizeY = image:getHeight() / 1024
			
			local object = utility.newQuad(sizeX , sizeY)
			object:setTexture(image)
			assetManager.listGui[nameKey] = object
			print(" Got gui " .. nameKey)
		end
		print("")
	end

-- Load avilable props
	function assetManager.loadProps()
		print("Loading props")
		local listFileNames = love.filesystem.getDirectoryItems("art/props/")
		
		for _ , fileName in pairs(listFileNames) do
			local nameKey = string.sub(fileName , 1 , #fileName - 4)
			local image = love.graphics.newImage("art/props/" .. fileName)
				
			local sizeX = image:getWidth() / 64
			local sizeY = image:getHeight() / 64
			
			local object = utility.newQuad(sizeX , sizeY)
			object:setTexture(image)
			assetManager.listProps[nameKey] = object
			print(" Got prop " .. nameKey)
		end
		print("")
	end
	
-- Load and make animation sets
	function assetManager.loadAnimations()
		print("Loading animations")
		local listFileNames = love.filesystem.getDirectoryItems("animations")
		
		for _ , fileName in pairs(listFileNames) do
			local nameKey = string.sub(fileName , 1 , #fileName - 10)
			local data = love.filesystem.load("animations/" .. fileName)()
			local image = love.graphics.newImage("art/animations/" .. nameKey .. ".png")
			
		-- Generate actual animation set
			local object = {}
			
			object.fps = data.fps
			object.spf = 1 / data.fps
			object.size = vector:new(image:getWidth() / data.cellWidth / 64 , image:getHeight() / data.cellHeight / 64)
			object.sequence = {}

		-- Meshes
			local offsetX , offsetY = 0 , 0
			for i , sequence in pairs(data.sequence) do
				object.sequence[i] = {}
				
				for j , frame in pairs(sequence) do	
					if frame[5] then
						offsetX = frame[5]
						offsetY = frame[6]
					end
				
					object.sequence[i][j] = utility.newFrame((frame[3] - frame[1]) * object.size.x , (frame[4] - frame[2]) * object.size.y , frame[1] / data.cellWidth , frame[2] / data.cellHeight , frame[3] / data.cellWidth , frame[4] / data.cellHeight , offsetX , offsetY)
					object.sequence[i][j]:setImage(image)
				end
			end
			
			assetManager.listAnimations[nameKey] = object
			
			print(" Got animation set " .. nameKey .. " with ".. #object.sequence .. " sequences")
		end
		
		print("")
	end
	
-- Load available entity classes
	function assetManager.loadEntities()
		print("Loading entities")
		local listFileNames = love.filesystem.getDirectoryItems("entities")
		
		for _ , fileName in pairs(listFileNames) do
			local nameKey = string.sub(fileName , 1 , #fileName - 7)
			assetManager.listEntities[nameKey] = love.filesystem.load("entities/" .. fileName)()
			setmetatable(assetManager.listEntities[nameKey] , entity)
			print(" Got entity " .. nameKey)
		end
		print("")
	end
	
-- Load available components
	function assetManager.loadComponents()
		print("Loading components")
		local listFileNames = love.filesystem.getDirectoryItems("components")
		local bit = 1
		
		for _ , fileName in pairs(listFileNames) do
			local nameKey = string.sub(fileName , 1 , #fileName - 10)
			local object = love.filesystem.load("components/" .. fileName)()
			setmetatable(object , component)
			object.bit = bit
			bit = bit * 2
			assetManager.listComponents[nameKey] = object
			print(" Got component " .. nameKey .. " : bit " .. object.bit)
		end
		print("")
	end
	
---- Load all assets
	function assetManager.load()
		assetManager.loadGui()
		assetManager.loadProps()
		assetManager.loadAnimations()
		assetManager.loadComponents()
		assetManager.loadEntities()
	end
	
