---- Entity prototype
	require "transform"
	
	entity = {}
	entity.__index = entity
	
---- Transform calls
-- Translate
	function entity:translate(...)
		self.transform:translate(...)
	end
	
-- Set position
	function entity:setPosition(...)
		self.transform:setPosition(...)
	end
	
-- Rotate
	function entity:rotate(...)
		self.transform:rotate(...)
	end
	
-- Set rotation
	function entity:setRotation(...)
		self.transform:setRotation(...)
	end

-- Scale
	function entity:scaleBy(...)
		self.transform:scaleBy(...)
		self:setSize()
	end

-- Set scale
	function entity:setScale(...)
		self.transform:setScale(...)
		self:setSize()
	end

---- Component functions
-- Add component
	function entity:addComponent(name , ...)
		create.component(name , self , ...)
	end
	
-- Get component
	function entity:getComponent(name)
		return self.components[name] or nil
	end
	
-- Remove component
	function entity:removeComponent(name)
		if self.components[name] then
			self.components[name] = nil
		end
		self:makeKey()
	end
	
-- Component key
	function entity:makeKey()
		self.key = 0
		for _ , component in pairs(self.components) do
			self.key = bit.bor(self.key , component.bit)
		end
		print(" Entity " .. self.id .. " has key " .. self.key)
	end
	
---- Hierarchy functions
-- Add child
	function entity:addChild(targetEntity)		
		if targetEntity.parent then
			targetEntity.parent:removeChild(targetEntity)
		end
		
		table.insert(self.children , targetEntity)
		targetEntity.parent = self
		targetEntity.idChild = #self.children
		
		--print("Entity " .. self.id .. " has " .. #self.children .. " children")
	end
	
-- Remove child
	function entity:removeChild(targetEntity)
		if self.children[targetEntity.idChild] then
			self.children[targetEntity.idChild].parent = nil
			table.remove(self.children , targetEntity.idChild)
			
			for i , child in pairs(self.children) do
				child.idChild = i
			end
			
			--print("Entity " .. self.id .. " has " .. #self.children .. " children")
		end
	end
	
-- Set parent
	function entity:setParent(targetEntity)
		targetEntity:addChild(self)
	end

-- Clear parent
	function entity:clearParent()
		self.parent:removeChild(self)
	end
	
-- Destroy entity and children
	function entity:destroy()
		print(" Entity " .. self.id .. " to be destroyed")
		self:clearParent()
		self = nil
		print("  Done")
	end

---- Attribute functions
-- Set size and radius
	function entity:setSize(x , y)
		x = x or self.size.x
		y = y or x or self.size.y
		self.size.x = x * self.transform.scale.x
		self.size.y = y * self.transform.scale.y
		
		self.radius = math.max(self.size.x , self.size.y) * 0.5
		
		self:updateRegion()
	end

-- Update region
	local x , y
	function entity:updateRegion()
		x = self.size.x * 0.5
		y = self.size.y * 0.5

		self.boxMin.x = -x + self.transform.positionWorld.x
		self.boxMin.y = -y + self.transform.positionWorld.y
		self.boxMax.x = x + self.transform.positionWorld.x
		self.boxMax.y = y + self.transform.positionWorld.y
	end
	
---- Constructor
	function entity:new()
		local object = {}
		
		setmetatable(object , self)

	-- Containers and pointers
		object.parent = nil
		object.children = {}
		object.components = {}
						
	-- Transform
		object.transform = transform:new(object)
		
	-- Attributes
		object.radius = 1
		object.size = vector:new(1 , 1)
		object.boxMin = vector:new(-0.5 , -0.5)		
		object.boxMax = vector:new(0.5 , 0.5)
		
		return object
	end
	
---- GUI entity
	entityGUI = {}
	entityGUI.__index = entity
	
---- Constructor
	function entityGUI:new()
		local object = {}
		
		setmetatable(object , self)
		
	-- Containers and pointers
		object.parent = nil
		object.children = {}
		object.components = {}
						
	-- Transform
		object.transform = transformGUI:new(object)
		
	-- Attributes
		object.radius = 1
		object.size = vector:new(1 , 1)
		object.boxMin = vector:new(-0.5 , -0.5)		
		object.boxMax = vector:new(0.5 , 0.5)
		
		return object
	end
