---- Generic math functions
-- Mix
	function math.mix(min , max , value)
		return (max - min) * value + min
	end
	
-- Clamp
	function math.clamp(min , max , value)
		return math.min(math.max(value , min) , max)
	end
	
-- Distance
	function math.distance(x1 , y1 , x2 , y2)
		return math.sqrt((x2 - x1)^2 + (y2 - y1)^2)
	end
	
-- Sign
	function math.sign(value)
		if value < 0 then
			return -1
		elseif value > 0 then
			return 1
		else
			return 0
		end
	end

---- Lines
-- Intersection between line and perpendicular line defined by point
	local m1 , m2 , b1 , b2 , xOut , yOut , xDif , yDif
	function math.linePointIntersect(point , vertexOne , vertexTwo)	
		xDif = vertexTwo.x - vertexOne.x
		yDif = vertexTwo.y - vertexOne.y
		
	-- If line is horizontal
		if yDif == 0 then
			return point.x , vertexOne.y
	-- If line is vertical
		elseif xDif == 0 then
			return vertexOne.x , point.y
		end
	
	-- Define line
		m1 = yDif / xDif
		b1 = vertexOne.y - (m1 * vertexOne.x)
	-- Define perpendicular line
		m2 = -1 / m1
		b2 = -(m2 * point.x) + point.y

	-- Intersection point
		xOut = (b1 - b2) / (m2 - m1)
		yOut = (m1 * xOut) + b1

		return xOut , yOut
	end

-- Line segment intersection, does not detect parallel
	local a1 , c1 , a2 , c2 , denominator
	function math.lineIntersect(vertexOne , vertexTwo , vertexThree , vertexFour)
	-- Define line 1
		a1 = vertexTwo.y - vertexOne.y
		b1 = vertexOne.x - vertexTwo.x
		c1 = a1 * vertexOne.x + b1 * vertexOne.y
		
	-- Define line 2
		a2 = vertexFour.y - vertexThree.y
		b2 = vertexThree.x - vertexFour.x
		c2 = a2 * vertexThree.x + b2 * vertexThree.y
		
		denominator = a1 * b2 - a2 * b1

	-- Point of intersection
		xOut = (b2 * c1 - b1 * c2) / denominator
		yOut = (a1 * c2 - a2 * c1) / denominator
	
		return xOut , yOut
	end
	
-- Line and AABB intersection
	local fracMin , fracMax
	local hasHit = false
	function math.lineAABBIntersect(vertexOne , vertexTwo , boxMin , boxMax)
	-- Difference between vertices
		xDif = vertexTwo.x - vertexOne.x
		yDif = vertexTwo.y - vertexOne.y
	
	-- Fraction of x
		hasHit , fracMin , fracMax = math.lineFraction(vertexOne.x , boxMin , boxMax , "x" , xDif , 0 , 1)
	
	-- Fraction of y if there was no miss in x
		if hasHit then
			hasHit , fracMin , fracMax = math.lineFraction(vertexOne.y , boxMin , boxMax , "y" , yDif , fracMin , fracMax)
	
		-- If still not missed then return intersection point
			if hasHit then
				return true , vertexOne.x + xDif * fracMin , vertexOne.y + yDif * fracMin
			end
		end
		
		return false
	end
	
-- Line fraction check
	local fracMinDim , fracMaxDim
	function math.lineFraction(vertexDim , boxMin , boxMax , axis , difference , fracMin , fracMax)
	-- Fraction
		fracMinDim = (boxMin[axis] - vertexDim) / difference
		fracMaxDim = (boxMax[axis] - vertexDim) / difference
		
	-- Swap if min is higher than max
		if fracMaxDim < fracMinDim then
			fracMaxDim , fracMinDim = fracMinDim , fracMaxDim
		end
		
	-- Check fraction is within clip range
		if fracMaxDim < fracMin then
			return false
		end
		
		if fracMinDim > fracMax then
			return false
		end
		
	-- If not missed then return true and new clip range
		return true , math.max(fracMinDim , 0) , math.min(fracMaxDim , 1)
	end
	
---- Vector math
-- Length table
	function math.length(table)
		if #table > 2 then
			return math.sqrt((table[1] ^ 2) + (table[2] ^ 2) + (table[3] ^ 2))
		else
			return math.sqrt((table[1] ^ 2) + (table[2] ^ 2))
		end
	end

-- Normalize table
	local length
	function math.normalizeTable(table)
		if #table > 2 then
			length = math.length(table)
			return {table[1] / length , table[2] / length , table[3] / length}
		else
			length = math.length(table)
			return {table[1] / length , table[2] / length}
		end
	end

-- Dot table
	function math.dotTable(a , b)
		if #a > 2 then
			return (a[1] * b[1]) + (a[2] * b[2]) + (a[3] * b[3])
		else
			return (a[1] * b[1]) + (a[2] * b[2])
		end
	end
