---- Vector prototype
	vector = {}
	vector.__index = vector

---- Arithmetic	
-- Add
	function vector:__add(other)
		if type(other) == "number" then
			if self.z then
				return vector:new(self.x + other , self.y + other , self.z + other)
			else
				return vector:new(self.x + other , self.y + other)
			end
		else
			if self.z and other.z then
				return vector:new(self.x + other.x , self.y + other.y , self.z + other.z)
			else
				return vector:new(self.x + other.x , self.y + other.y)
			end
		end
	end
	
-- Subtract
	function vector:__sub(other)
		if type(other) == "number" then
			if self.z then
				return vector:new(self.x - other , self.y - other , self.z - other)
			else
				return vector:new(self.x - other , self.y - other)
			end
		else
			if self.z then
				return vector:new(self.x - other.x , self.y - other.y , self.z - other.z)
			else
				return vector:new(self.x - other.x , self.y - other.y)
			end
		end
	end
		
-- Multiply
	function vector:__mul(other)
		if type(other) == "number" then
			if self.z then
				return vector:new(self.x * other , self.y * other , self.z * other)
			else
				return vector:new(self.x * other , self.y * other)
			end
		else
			if self.z then
				return vector:new(self.x * other.x , self.y * other.y , self.z * other.z)
			else
				return vector:new(self.x * other.x , self.y * other.y)
			end
		end
	end
	
-- Divide
	function vector:__div(other)
		if type(other) == "number" then
			if self.z then
				return vector:new(self.x / other , self.y / other , self.z / other)
			else
				return vector:new(self.x / other , self.y / other)
			end
		else
			if self.z then
				return vector:new(self.x / other.x , self.y / other.y , self.z / other.z)
			else
				return vector:new(self.x / other.x , self.y / other.y)
			end
		end
	end

---- Math functions
-- Length
	function vector:length()
		if self.z then
			return math.sqrt(self.x^2 + self.y^2 + self.z^2)
		else
			return math.sqrt(self.x^2 + self.y^2)
		end
	end
	
-- Distance
	function vector:distance(otherVector)
		if self.z then
			return math.sqrt((self.x - otherVector.x)^2 + (self.y - otherVector.y)^2 + (self.z - otherVector.z)^2)
		else
			return math.sqrt((self.x - otherVector.x)^2 + (self.y - otherVector.y)^2)
		end
	end

-- Normalize
	function vector:normalize()
		local length = self:length()
		
		if self.z then
			self.x = self.x / length
			self.y = self.y / length
			self.z = self.z / length
		else
			self.x = self.x / length
			self.y = self.y / length
		end
	end
	
-- Dot
	function vector:dot(other)
		if self.z then
			return self.x * other.x + self.y * other.y + self.z * other.z
		else
			return self.x * other.x + self.y * other.y
		end
	end
	
-- Cross
	function vector:cross(other)
		if self.z then
			return vector:new((self.y * other.z) - (self.z * other.y) , (self.z * other.x) - (self.x * other.z) , (self.x * other.y) - (self.y * other.x))
		else
			return (self.x * other.y) - (self.y * other.x)
		end
	end

---- Utility functions
-- Print
	function vector:print()
		if self.z then
			print("(" .. self.x .. " , " .. self.y .. " , " .. self.z .. ")")
		else
			print("(" .. self.x .. " , " .. self.y .. ")")
		end
	end
	
-- Print to screen
	function vector:printToScreen(position)
		if self.z then
			love.graphics.print("(" .. self.x .. " , " .. self.y .. " , " .. self.z .. ")" , position.x , position.y)
		else
			love.graphics.print("(" .. self.x .. " , " .. self.y .. ")" , position.x , position.y)
		end
	end
	
-- Draw direction
	function vector:drawDirection(x , y , length)
		x = x or 0
		y = y or 0
		length = length or 1
		love.graphics.line(x , y , x + self.x * length , y + self.y * length)
	end

---- Constructor
	function vector:new(x , y , z)
		local object = {}
		
		setmetatable(object , self)
		self.__index = self
		
		object.x = x or 0
		object.y = y or 0
		
		if z then
			object.z = z
		end
		
		return object 
	end
