---- SVG scene importer
	sceneImport = {}
	
---- Utility functions
-- Find group by name in line
	function sceneImport.findGroup(line , string)
		if string.match(line , "<g") then
			if string.match(line , string) then
				return true
			else
				return false
			end
		else
			return false
		end
	end

-- Extract value in line after substring
	function sceneImport.extractValue(line , subString , stop , keepString)
		stop = stop or '"'
		
	-- Look for string
		local s , e = string.find(line , subString)
		
	-- If string was found, extract value
		if e then
			s = e + 1
			for i = s , #line do
			-- Stop at next stop mark
				if string.sub(line , i , i) == stop then
					break
				end
				e = i
			end
		-- Return as string
			if keepString then
				return string.sub(line , s, e)
		-- Return as number
			else
				return tonumber(string.sub(line , s, e))
			end
	-- If string was not found, return nil
		else
			return nil
		end
	end

---- Main import function, adds content to the world
	function sceneImport.import(file)
		print("Importing scene " .. file)
	
	-- Load file with scene info
		local sceneInfo = love.filesystem.load("/scenes/" .. file .. ".scene")()
		
	-- Load source xml and store in table
		local source = {}
		for line in love.filesystem.lines("/scenes/" .. sceneInfo.source .. ".svg") do
			table.insert(source , line)
		end
		
	-- Get scene size
		local sizeX , sizeY
		for _ , line in pairs(source) do
			if string.match(line , "<svg") then
				sizeX = sceneImport.extractValue(line , 'width="') * 0.01
				sizeY = sceneImport.extractValue(line , 'height="') * 0.01
				break
			end
		end
		
		world.rootEntity:setSize(sizeX , sizeY)
		print(" Scene size is " .. sizeX .. "x" .. sizeY)
		
	-- Find start and end lines of layer groups
		local foundScene = false
		local foundCollision = false
		local foundGame = false
		
		local sourceScene = {}
		local sourceCollision = {}
		local sourceGame = {}
		
		for i , line in pairs(source) do
			if sceneImport.findGroup(line , "scene") then
				foundScene = true
				foundCollision = false
				foundGame = false
				print(" Scene group starts at line " .. i)
			end
			
			if sceneImport.findGroup(line , "collision") then
				foundCollision = true
				foundScene = false
				foundGame = false
				print(" Collision group starts at line " .. i)
			end
			
			if sceneImport.findGroup(line , "game") then
				foundGame = true
				foundScene = false
				foundCollision = false
				print(" Game group starts at line " .. i)
			end
			
		-- Add lines to group source tables
			if foundScene then
				table.insert(sourceScene , line)
			end
			
			if foundCollision then
				table.insert(sourceCollision , line)
			end
			
			if foundGame then
				table.insert(sourceGame , line)
			end
		end
		
	-- Remove first and last line from source tables as they are only group tags
		table.remove(sourceScene , 1)
		table.remove(sourceScene , #sourceScene)
		print("  Scene source has " .. #sourceScene .. " lines")
		
		table.remove(sourceCollision , 1)
		table.remove(sourceCollision , #sourceCollision)
		print("  Collision source has " .. #sourceCollision .. " lines")
		
		table.remove(sourceGame , 1)
		table.remove(sourceGame , #sourceGame)
		table.remove(sourceGame , #sourceGame)
		print("  Game source has " .. #sourceGame .. " lines")
		
	-- Clear main source
		source = nil
		
		print("")
		
	-- Scene
		print(" Getting scene assets")
		local posX , posY , posZ , scaleX , scaleY , rotX , rotY
		local nameImage
		
		local foundGroup = false
		posZ = 0
		for _ , line in pairs(sourceScene) do
		-- Check if new z-group
			if string.match(line , "<g") then
				posZ = sceneImport.extractValue(line , 'id="')
		-- Else get entity data from line
			elseif not string.match(line , "</g") then
			-- Rotation
			-- Look for matrix
				if string.match(line , 'matrix%(') then
				-- Find start point of matrix
					local s , e = string.find(line , 'matrix%(')
					s = e + 1
					
				-- Find endpoint of first value
					for i = s , #line do
						if string.sub(line , i , i) == "," then
							e = i - 1
							break
						end
					end
					rotX = tonumber(string.sub(line , s , e))
					
				-- Find endpoint of second value
					s = e + 2
					for i = s , #line do
						if string.sub(line , i , i) == "," then
							e = i - 1
							break
						end
					end
					rotY = -tonumber(string.sub(line , s , e))
			-- Else look for scale in case object was rotated 180 degrees
				elseif string.match(line , 'scale%(') then
					rotX = -1
					rotY = 0
			-- If no matrix, assign default rotation
				else
					rotX = 1
					rotY = 0
				end
				
			-- Image
				nameImage = sceneImport.extractValue(line , "/props/" , "." , true)
				
				if not nameImage or not assetManager.listProps[nameImage] then
					nameImage = "notFound"
				end
				
				local sizeImageX , sizeImageY = utility.getQuadSize(assetManager.listProps[nameImage])
				
				local width = sceneImport.extractValue(line , 'width="')
				local height = sceneImport.extractValue(line , 'height="')
				
			-- Scale
				scaleX = (width * 0.01) / sizeImageX
				scaleY = (height * 0.01) / sizeImageY
				
			-- Position, adjusted to image center	
				local xSVG = sceneImport.extractValue(line , 'x="') + (width * 0.5)
				local ySVG = -sceneImport.extractValue(line , 'y="') - (height * 0.5)
				
				posX = (xSVG * rotX + ySVG * -rotY) * 0.01
				posY = sizeY + (xSVG * rotY + ySVG * rotX) * 0.01

			-- Add to scene
				tempObject = create.entity("prop" , nameImage)
				tempObject:translate(posX , posY , posZ)
				tempObject:setRotation(rotX , rotY)
				tempObject:setScale(scaleX , scaleY)
				tempObject.transform:parentTransform()
				tempObject.transform:makeMatrixTransform()
				tempObject.transform:makeMatrixToShader()
				tempObject:updateRegion()
			
				print("  Added scene prop using image " .. nameImage)
			end
		end
		
		print("")
	
	-- Collision
		print(" Getting collision")
		local vertices = {}
		local vertexString , char
		local s , e , xTotal , yTotal , count , xMin , xMax , yMin , yMax , thick
		
		foundGroup = false
		posZ = 0
		for _ , line in pairs(sourceCollision) do
		-- Z value
			posZ = sceneImport.extractValue(line , 'z="')
			if not posZ then
			 posZ = 0
			end
			
		-- Find start and end for vertex string
			s , e = string.find(line , ' d="')
			s = e + 3
			for i = s , #line do
				if string.sub(line , i , i) == '"' then
					e = i - 2
					break
				end
			end
		
		-- Get vertex string
			vertexString = string.sub(line , s , e)
			--print("Vertex string: " .. vertexString)
	
		-- Reset vertex list
			vertices = {}
			vertices[1] = {}
			e = 0
			s = 1

		-- Extract values from vertex string
			for i = 1 , #vertexString do
			-- Check current character
				char = string.sub(vertexString , i , i)
	
			-- X value
				if char == "," then
					vertices[#vertices][1] = tonumber(string.sub(vertexString , s , s + e - 1)) * 0.01
					--print(s .. " " .. e)
					--print("X " .. vertices[#vertices][1])
					s = i + 1
					e = 0
			-- Y value
				elseif char == " " then
					vertices[#vertices][2] = sizeY - tonumber(string.sub(vertexString , s , s + e - 1)) * 0.01
					--print(s .. " " .. e)
					--print("Y " .. vertices[#vertices][2])
					s = i + 1
					e = 0
					if i ~= #vertexString then
						vertices[#vertices + 1] = {}
					end
				else
					e = e + 1
				end
			end
		
		-- Calculate position of shape
			xMin , xMax , yMin , yMax = vertices[1][1] , vertices[1][1] , vertices[1][2] , vertices[1][2]
			for _ , vertex in pairs(vertices) do
				xMin = math.min(xMin , vertex[1])
				xMax = math.max(xMax , vertex[1])
				yMin = math.min(yMin , vertex[2])
				yMax = math.max(yMax , vertex[2])
			end
	
			posX = math.mix(xMin , xMax , 0.5)
			posY = math.mix(yMin , yMax , 0.5)

		-- Make vertex values local
			for _ , vertex in pairs(vertices) do
				vertex[1] = vertex[1] - posX
				vertex[2] = vertex[2] - posY
			end
			
		-- Find thickness value
			thick = sceneImport.extractValue(line , 'thick="')
			if not thick then
				thick = 0
			end

		-- Add to scene
			tempObject = create.entity("sceneShape" , vertices)
			tempObject.components.collider.thick = thick
			tempObject:translate(posX , posY , posZ)
			tempObject.transform:parentTransform()
			tempObject.transform:makeMatrixTransform()
			tempObject.transform:makeMatrixToShader()
			tempObject.components.collider:toWorld()
			tempObject.components.collider:toSize()
		
			print("  Added shape with " .. #vertices .. " vertices")
		end
		
		print("")
		
	-- Game
		print(" Getting game entities")
		foundGroup = false
		local entityName
		for _ , line in pairs(sourceGame) do
		-- Get entity data
			entityName = sceneImport.extractValue(line , 'entity="' , '"' , true)
			
		-- Proceed if name was found
			if entityName then
				print("  Entity " .. entityName .. " found")
				
				posX = (sceneImport.extractValue(line , ' x="') + sceneImport.extractValue(line , 'width="') * 0.5) * 0.01
				posY = world.rootEntity.size.y - (sceneImport.extractValue(line , ' y="') + sceneImport.extractValue(line , 'height="') * 0.5) * 0.01
				
			-- Create entity
				tempObject = create.entity(entityName)
				tempObject:setPosition(posX , posY)
				
				if entityName == "player" then
					world.player = tempObject
					world.camera:setParent(world.player)
				end
			end
		end
		
		print("")
		print("Scene import complete")
	end
