---- System manager
	require "system"

	systemManager = {}
	
	systemManager.listSystems = {{"timed" , "control" , "ai" , "movement" , "transform" , "collisionUpdate" , "grid" , "projectile" , "combat" , "collision" , "transform" , "transformToScreen" , "animation"} , {"draw" , "drawGUI"}}
	
	systemManager.shaderDebug = love.graphics.newShader("shaders/default.vertex" , "shaders/unlit.pixel")
	
---- Load systems
	function systemManager.load()
		print("Loading systems")
		local listFileNames = love.filesystem.getDirectoryItems("systems/")

		for _ , fileName in pairs(listFileNames) do
			local nameKey = string.sub(fileName , 1 , #fileName - 7)
			local object = table.merge(system:new() , love.filesystem.load("systems/" .. fileName)())
			
			setmetatable(object , system)
	
		-- Generate bit lock for system
			object.lock = 0
			for _ , name in pairs(object.require) do
				if assetManager.listComponents[name] then
					object.lock = bit.bor(object.lock , assetManager.listComponents[name].bit)
				end
			end

		-- Put system in list
			local found = false
			for i , subList in pairs(systemManager.listSystems) do
				for j , name in pairs(subList) do
					if nameKey == name then
						systemManager.listSystems[i][j] = object
						found = true
					end
				end
			end
			
			print(" Got system " .. nameKey .. " : lock " .. object.lock)
			if not found then
				print("  System has no designated slot")
			else
				object.name = nameKey
			end
		end
		print("")
		print("Systems will be executed in the following order")
		print(" Update")
		for _ , system in pairs(systemManager.listSystems[1]) do
			print("  " .. system.name)
		end
		print("")
		print(" Draw")
		for _ , system in pairs(systemManager.listSystems[2]) do
			print("  " .. system.name)
		end
		print("")
	end

-- Initiate systems
	function systemManager.init()
		for _ , system in pairs(systemManager.listSystems[1]) do
			system:init()
		end
		for _ , system in pairs(systemManager.listSystems[2]) do
			system:init()
		end
	end

---- Callbacks
-- Update
	function systemManager.update()
		for _ , system in pairs(systemManager.listSystems[1]) do
			system:first()
			
			for _ , child in pairs(world.rootEntity.children) do
				system:iterate(child)
			end
			
			system:last()
		end
	end

-- Draw
	function systemManager.draw()
		for _ , system in pairs(systemManager.listSystems[2]) do
			system:first()
			
			for _ , child in pairs(world.rootEntity.children) do
				system:iterate(child)
			end
			
			system:last()
		end
		
		if isDebug then
			systemManager.debug()
		end
	end
	
-- Debug
	function systemManager.debug()
		for _ , system in pairs(systemManager.listSystems[2]) do
			love.graphics.setShader()
			system:firstDebug()
			
			love.graphics.setShader(systemManager.shaderDebug)
			for _ , child in pairs(world.rootEntity.children) do
				system:iterateDebug(child)
			end
			
			love.graphics.setShader()
			system:lastDebug()
		end
		
		love.graphics.setLineWidth(0.015)
		
		for _ , system in pairs(systemManager.listSystems[1]) do
			love.graphics.setShader()
			system:firstDebug()
			
			love.graphics.setShader(systemManager.shaderDebug)
			for _ , child in pairs(world.rootEntity.children) do
				system:iterateDebug(child)
			end
			
			love.graphics.setShader()
			system:lastDebug()
		end
		
		love.graphics.setColor(color.white)
	end
