---- Transform
	transform = {}
	transform.__index = transform

---- Basic transformation	
-- Translate
	function transform:translate(x , y , z)	
		x = x or 0
		y = y or 0
		
		self.position.x = self.position.x + x
		self.position.y = self.position.y + y
		
		if z then
			self.position.z = self.position.z + z
			self:makeMatrixDepth()
		end
		
		self.hasChanged = true
	end

-- Set position
	function transform:setPosition(x , y , z)
		self.position.x = x or 0
		self.position.y = y or 0
		
		if z then
			self.position.z = z
			self:makeMatrixDepth()
		end
		
		self.hasChanged = true
	end
	
-- Rotate
	local rotX , rotY , cosValue , sinValue
	function transform:rotate(angle)
		angle = math.rad(angle)
		
		cosValue = math.cos(angle)
		sinValue = math.sin(angle)
		
		self.rotation.x = self.rotation.x * cosValue + self.rotation.y * -sinValue
		self.rotation.y = self.rotation.x * sinValue + self.rotation.y * cosValue
	
		self.hasChanged = true
	end	
	
-- Set rotation
	function transform:setRotation(x , y)
		self.rotation.x = x
		self.rotation.y = y
		
		self.isChanged = true
	end
	
-- Scale
	function transform:scaleBy(x , y)
		self.scale.x = self.scale.x * x
		self.scale.y = self.scale.y * (y or x)
		
		self.hasChanged = true
	end
	
-- Set scale
	function transform:setScale(x , y)
		self.scale.x = x
		self.scale.y = y or x
		
		self.hasChanged = true
	end
	
-- Parent transform
	local x , y
	function transform:parentTransform()
		x = self.position.x * self.parent.parent.transform.scaleWorld.x
		y = self.position.y * self.parent.parent.transform.scaleWorld.y
		
	-- Position
		self.positionWorld.x = x * self.parent.parent.transform.rotationWorld.x + y * -self.parent.parent.transform.rotationWorld.y + self.parent.parent.transform.positionWorld.x
		self.positionWorld.y = x * self.parent.parent.transform.rotationWorld.y + y * self.parent.parent.transform.rotationWorld.x + self.parent.parent.transform.positionWorld.y
		self.positionWorld.z = self.position.z + self.parent.parent.transform.positionWorld.z
		
	-- Rotation
		self.rotationWorld.x = self.rotation.x * self.parent.parent.transform.rotationWorld.x + self.rotation.y * -self.parent.parent.transform.rotationWorld.y
		self.rotationWorld.y = self.rotation.x * self.parent.parent.transform.rotationWorld.y + self.rotation.y * self.parent.parent.transform.rotationWorld.x
		
	-- Scale
		self.scaleWorld.x = self.scale.x * self.parent.parent.transform.scaleWorld.x
		self.scaleWorld.y = self.scale.y * self.parent.parent.transform.scaleWorld.y
	end

---- Matrix functions
-- Depth matrix
	local scale
	function transform:makeMatrixDepth()
		scale = 1 / (self.positionWorld.z * 0.02 + 1)
		self.matrixDepth[1] = {scale , 0 , 0}
		self.matrixDepth[2] = {0 , scale , 0}
	end
	
-- Combined transform matrix
	function transform:makeMatrixTransform()		
		self.matrixTransform[1] = {self.scaleWorld.x * self.rotationWorld.x , self.scaleWorld.x * self.rotationWorld.y , 0}
		self.matrixTransform[2] = {self.scaleWorld.y * -self.rotationWorld.y , self.scaleWorld.y * self.rotationWorld.x , 0}
		self.matrixTransform[3] = {self.positionWorld.x , self.positionWorld.y , 1}
	
		--self.matrixTransform = (matrix:new({{self.scale.x , 0 , 0} , {0 , self.scale.y , 0} , {0 , 0 , 1}}) * matrix:new({{self.rotation.x , -self.rotation.y , 0} , {self.rotation.y , self.rotation.x , 0} , {0 , 0 , 1}})) * matrix:new({{1 , 0 , self.positionWorld.x} , {0 , 1 , self.positionWorld.y} , {0 , 0 , 1}})
		
		--self.matrixTransform:print()
		--print("")
	end
	
-- Combined transform and projection matrix
	function transform:makeMatrixToShader()
		self.matrixToShader = self.matrixTransform * world.camera.matrixTransform * world.camera.matrixProjection * self.matrixDepth
	end

---- Constructor
	function transform:new(parent)
		local object = {}

		setmetatable(object , self)

		object.parent = parent
	
	-- Vectors
		object.position = vector:new(0 , 0 , 0)
		object.rotation = vector:new(1 , 0)
		object.scale = vector:new(1 , 1)
		
		object.positionWorld = vector:new(0 , 0 , 0)
		object.rotationWorld = vector:new(1 , 0)
		object.scaleWorld = vector:new(1 , 1)
		
	-- Matrices
		object.matrixTransform = matrix:new()
		object.matrixDepth = matrix:new()
		object.matrixToShader = matrix:new()
		
	-- State
		object.hasChanged = true
	
		return object
	end
	
---- GUI transform
	transformGUI = {}
	setmetatable(transformGUI , transform)
	transformGUI.__index = transformGUI
	
-- Screen space transform matrix
	function transformGUI:makeMatrixToShader()
		self.matrixToShader = self.matrixTransform * world.camera.matrixProjectionGUI
	end
	
---- Constructor
	function transformGUI:new(parent)
		local object = {}
		
		setmetatable(object , self)
		
		object.parent = parent
	
	-- Vectors
		object.position = vector:new(0 , 0 , 0)
		object.rotation = vector:new(1 , 0)
		object.scale = vector:new(1 , 1)
		
		object.positionWorld = vector:new(0 , 0 , 0)
		object.rotationWorld = vector:new(1 , 0)
		object.scaleWorld = vector:new(1 , 1)
		
	-- Matrices
		object.matrixTransform = matrix:new()
		object.matrixToShader = matrix:new()
		
	-- State
		object.hasChanged = true
	
		return object
	end
